#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Script Name       : beautify.py
# Author            : mutsx
# Created           : Mon, 26 Oct 2015
# Last Modified     : Fri, 06 Nov 2015
# License           : MIT


import os
import fnmatch
import shutil
import re
from prettytable import PrettyTable
from mubu.progressbar import progress
from mubu.mutagen import tags
from mubu.previewer import preview

class Beautifier:

    def __init__(self, exceptions, beautify, remove_original, recursive, clear, only):
        self.exceptions      = exceptions
        self.exceptions      = exceptions if exceptions else []
        self.beautify        = beautify
        self.remove_original = remove_original
        self.recursive       = recursive
        self.clear           = clear
        self.only            = only[0].lower() if only else None
        self.destination     = '!mubu'
        self.corrected       = 0

    def get_files(self):
        '''
        Get all the files in the cwd

        :returns: array
        '''
        files = []
        if self.recursive:
            for root, dirnames, filenames in os.walk(os.getcwd()):
                for filename in filenames:
                    if filename.endswith(self.only or ('.mp3', '.ogg', '.flac')):
                        cwd = root.replace(os.getcwd(), '')
                        dirname = cwd[1::].split('/', 1)[0]
                        if dirname != self.destination:
                            file = os.path.join(cwd[1::], filename)
                            files.append(file)
        else:
            for file in os.listdir(os.getcwd()):
                if file.endswith(self.only or ('.mp3', '.ogg', '.flac')):
                    files.append(file)

        if len(files) < 1:
            exit('\033[0;31m[ ERROR ]\033[0m No {}-files in your current working directory.'.format(self.only or 'mp3/ogg/flac'))
        else:
            return sorted(files)

    def fix_unwanted_chars(self, text, chars):
        '''
        Fix characters in an already-title-cased string

        :param text: Part from the filename(s) (artist/title)
        :returns: string
        '''
        lower = False
        fixed_text = ''
        for letter in text:
            if lower and letter not in chars:
                fixed_text += letter.lower()
                lower = False
            elif letter in chars:
                lower = True
                fixed_text += letter
            else:
                fixed_text += letter
        return fixed_text

    def title_case(self, text):
        '''
        Title case a string

        :param text: Part from the filename(s) (artist/title)
        :returns: string
        '''
        name = text.replace('_', ' ').strip()
        if len(self.exceptions) > 0:
            #titled_text = self.title_case_with_exception(name)
            for exception in self.exceptions:
                if exception == name:
                    titled_text = name
                    break
                else:
                    titled_text = name.title()
        else:
            titled_text = name.title()
        return self.fix_unwanted_chars(titled_text, "0123456789'")


    def check_corrections(self, oldfile, newfile):
        '''
        Checks for a correction of the filename

        :param oldfile: The file in the cwd
        :param newfile: The old file in the new destination and corrected
        '''
        if oldfile != newfile:
            self.corrected += 1

    def print_results(self, total_files):
        '''
        Print the amount of files that has been corrected

        :param total_files: Integer that is the total amount of files
        '''
        result = PrettyTable(['Result(s)'])
        result.padding_width = 2
        result.max_width = 50
        result.add_row(['Amount of files corrected: {} / {}'.format(self.corrected, total_files)])
        print(result)

    def run(self):
        ''' Beautify the files '''
        total_runs = 0
        files = self.get_files()

        for file in files:
            filepath, filename = file.rsplit('/', 1) if self.recursive and '/' in file else ['', file]

            pattern  = '(?:[\d]{1,2}\s?[\.|\-\s])?'     # numbers  like 01. or 01 -
            pattern += '(.+?)\s*-\s*(.*)'               # artist - title
            pattern += '(\.(mp3|ogg|flac))'             # extension
            regex    = re.match(pattern, filename)

            try:
                artist      = self.title_case(regex.group(1))
                title       = self.title_case(regex.group(2))
                extension   = regex.group(4)
                root        = filepath + '/' if filepath else ''
                new_file    = '{} - {}.{}'.format(artist, title, extension) if not self.recursive else '{}{} - {}.{}'.format(root, artist, title, extension)
                file_dest   = os.path.join(self.destination, new_file)
            except AttributeError:
                files.remove(file)
                print('\033[0;31m[ ERROR ]\033[0m Please, specify \033[0;34m{}\033[0m as: ARTIST - TITLE'.format(file))
                continue

            if self.beautify:
                if not os.path.exists(self.destination):
                    os.makedirs(self.destination)
                if self.recursive and not os.path.exists(os.path.join(self.destination, filepath)):
                    os.makedirs(os.path.join(self.destination, filepath))

                shutil.copyfile(file, file_dest)
                tags.add(file_dest, artist, title, extension, self.clear)
                self.check_corrections(file, new_file)
                total_runs += 1
                progress.update(files, total_runs)
                if self.remove_original: os.remove(file)
            else:
                self.check_corrections(file, new_file)
                preview.add(file, new_file)

        if not self.beautify:
            preview.show(len(files), self.corrected)
        else:
            self.print_results(len(files))
